#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <ctype.h>
#include <math.h>
#include <time.h>
#include <errno.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

#define PI M_PI

#define  sind(x)        sin(x*(M_PI/180.))
#define  cosd(x)        cos(x*(M_PI/180.))
#define  tand(x)        tan(x*(M_PI/180.))
#define asind(x)        (180./M_PI)*asin(x)
#define acosd(x)        (180./M_PI)*acos(x)
#define atand(x)        (180./M_PI)*atan(x)
#define atan2d(x,y)     (180./M_PI)*atan2((x),(y))

/* Size of data files */
#define CUT_RA 5.0   /* degrees */
#define CUT_DEC 6.0  /* degrees */
#define STEPS_RA   360/(int)CUT_RA
#define STEPS_DEC  180/(int)CUT_DEC

#define WFRAME_RA 0.0625 /* degrees */
#define WFRAME_DEC 0.0625 /* degrees */
#define NFRAMES_RA (int)(CUT_RA/WFRAME_RA)
#define NFRAMES_DEC (int)(CUT_DEC/WFRAME_DEC)

#define JDoffset  2450449.5    /* JD   of 00:00:00 Jan. 1, 1997 for 2MASS  */

#define LGB2P 13 /* longueur chaine codage data position */
#define LGB2M 12 /* longueur chaine codage data magnitude 
		    (sans byte sup. du codage bimode PSF) */
#define LGB2O 6  /* longueur chaine codage data USNO */

typedef struct {
  double ra, dec;
  double glon, glat;
  float err_maj, err_min;
  int err_ang;
  int extd_flg;
  int mp_flg;
  char isopt;
  char date[11];
  int scan;
  char hemis;
  long id;
  char psfbiJ, psfbiH, psfbiK;
  int mult_id;
} position;

typedef struct {
  int n_mchs;
  char id;
  float dist;
  int phi;
  float b_m;
  float r_m;
} optical;

typedef struct {
  float m;
  float msig;
  float msigcom;
  float snr;
  int rd_flg;
  int bl_flg;
  char cc_flg;
  float m_stdap;
  float msig_stdap;
  int ndet_flg1;
  int ndet_flg2;
  float psfchi;
} databand;

typedef struct {
  position pos;
  databand J;
  databand H;
  databand K;
  optical opt;
} enregistrement;

typedef struct {
  double ra;
  double dec;
  long dataPtr;
  int Xra;
  int Xdec;
  int Fra;
  int Fdec;
} tabindex;

struct noeud{
  enregistrement rec;         
  double r;
  struct noeud *gauche ;
  struct noeud *droit ; 
};

typedef struct {
  long size;
  long nbElt;
  long dataStart;
  long dataEnd;
  double dec; /* south west corner */
  double ra;
  double alinf, alsup, delinf, delsup;
} Frame;

typedef struct {
  double alinf, alsup, delinf, delsup;
} FileBound;

typedef struct {
  double raC;
  double decC;
  double r;
  double alinf, alsup, delinf, delsup;
} Cone;

/*  VARIABLES GLOBALES  */

#ifndef __2MASS_glob
#define __2MASS_glob
static long mrec = 100, matched, tested, truncated, filtered;
static double maxR;
int DEBUG;
int VERB;
int HTML;
#endif

/*  PROTOTYPES  */

void dminmax(double, double *, double *);
static double sphDst(double ,double ,double ,double );
static void eq2gal(double, double, double *, double *);
static int tr_uu(double [3], double [3], double [3][3]); 
static int tr_uo (double [3], double [2]);   
static int tr_ou (double [2], double [3]);  
char *makeName(position);
char *makeName2(position);

int isIn(double, double, double);
int isInGlob(double, double, double, double);
int isFileOnCone (FileBound, Cone);
int isFrameOnCone (Frame, Cone);

int testOf2(long long *, int, char *, char, long);

enregistrement read2MASS(char *);
void printEnr(enregistrement, FILE *);

void posCode(position, unsigned char *, long, double, double);
void magCode(databand, unsigned char *, char, long, int);
void optCode(optical, unsigned char *, long);

static position posDecode(unsigned char *, double, double);
static databand magDecode(unsigned char *, int);
static optical optDecode(unsigned char *);
static long sDecode(unsigned char [], int, int);
static long sDecode2(unsigned char [], int, int);

char *jd2date(double);
struct tm jd2tm(double);
double date2jd(char *);
double tm2jd(struct tm *);

struct noeud *ajoutarbre(struct noeud *, enregistrement, double);
void affarbre(struct noeud *);
struct noeud *allouernoeud();
double maxArbre(struct noeud *);

void nrerror(char *);
int *ivector(long , long );
long *lvector(long , long );
unsigned long *ulvector(long , long );
double *dvector(long , long );
position *posvector(long , long );
enregistrement *enrvector(long , long);
void free_ivector(int *, long , long);
void free_lvector(long *, long , long );
void free_ulvector(unsigned long *, long , long );
void free_dvector(double *, long , long );
void free_posvector(position *, long , long );
void free_enrvector(enregistrement *, long , long );


void indexxPtr(unsigned long , long [], unsigned long []);
